// Librairie graphique pour afficheur LCD GDM1284A sur MICROCHIP C PIC
// C.Dupaty 04-2003 modifie 02-2005
// ralise  partir d'une tude de Pascal Stang pour ATMEL AVR

// ATTENTION
// les coordonnes (0,0) reprsente le point en haut  gauche de l'afficheur
// avec le connecteur en bas

/*
void glcdInit(void)						// pour initialisation (obligatoire)
void glcdSetDot(unsigned char x, unsigned char y)	// affiche un point
void glcdClearDot(unsigned char x, unsigned char y)	// efface le point
void glcdLine(int x1, int y1, int x2, int y2,unsigned char coul)	// trace une ligne (noire si coul =1 blanche si coul=0)
void glcdRectangle(unsigned char x, unsigned char y, unsigned char a, unsigned char b,unsigned char coul)	//trace un rectangle vide (x,y)coin bas gauche (a,b) largeur et hauteur
void glcdFRectangle(unsigned char x, unsigned char y, unsigned char a, unsigned char b,unsigned char coul)	//trace un rectangle plein (x,y)coin bas gauche (a,b) largeur et hauteur
void glcdCircle(unsigned char xcenter, unsigned char ycenter, unsigned char radius,unsigned char coul) // trace un pseudo cercle

void glcdGotoChar(unsigned char line, unsigned char col) // positionne le curseur texte
void glcdWriteChar(char c)		// crit un caractre ASCII (police 5x7)
void glcdWriteCharGr(unsigned char grCharIdx)	// crit un caractre semi graphique
void glcdPutStr(rom unsigned char *data)	// crit une chaine de caractres en ROM
void glcdPutStrRAM(unsigned char *data)	// crit une chaine de caractres en RAM

void glcdStartLine(unsigned char start) // dfinit la ligne de dpart de l'afficheur (pour scrolling)
void glcdClearScreen(void)				// efface tout
void glcdDelay(unsigned int p)			// tempo
*/


// modifier les lignes ci dessous en fonction de la configuration matrielle

#define LCD_CTRL_PORT	PORTB		// PORT for LCD control signals
#define LCD_CTRL_DDR	TRISB		// DDR register of LCD_CTRL_PORT
#define LCD_CTRL_RS		PORTBbits.RB2		// pin for LCD Register Select
#define LCD_CTRL_RW		PORTBbits.RB1		// pin for LCD Read/Write
#define LCD_CTRL_E		PORTBbits.RB0		// pin for LCD Enable
#define LCD_CTRL_CS0	PORTBbits.RB3		// pin for LCD Cntrlr 0 Chip Select
#define LCD_CTRL_CS1	PORTBbits.RB4		// pin for LCD Cntrlr 1 Chip Select
#define LCD_CTRL_RESET	PORTBbits.RB5		// pin for LCD Reset
#define LCD_DATA_PORT	PORTD		// PORT for LCD data signals
#define LCD_DATA_DDR	TRISD		// DDR register of LCD_DATA_PORT

#define GLCD_ON_CTRL		0x3E	// 0011111X: lcd on/off control
#define GLCD_ON_DISPLAY		0x01	//		DB0: turn display on
#define GLCD_START_LINE		0xC0	// 11XXXXXX: set lcd start line
#define GLCD_SET_PAGE		0xB8	// 10111XXX: set lcd page (X) address
#define GLCD_Y_ADDR			0x40	// 01YYYYYY: set lcd Y address
#define GLCD_STATUS_BUSY	0x80	// (1)->LCD IS BUSY
#define GLCD_STATUS_ONOFF	0x20	// (0)->LCD IS ON
#define GLCD_STATUS_RESET	0x10	// (1)->LCD IS RESET

// LCD geometry defines 
#define GLCD_XPIXELS			128		// pixel width of entire display
#define GLCD_YPIXELS			64		// pixel height of entire display
#define GLCD_CONTROLLER_WIDTH	64		// pixel width of one display controller

#define FALSE	0
#define TRUE	-1
#define entre 	0xFF	// pour DDR PORT E/S
#define sort	0x00

#define d1		0x1000	// tempo "1s" 
#define d2		0x2000	// tempo "2s"

// typedefs/structures
typedef struct struct_GrLcdCtrlrStateType
{
	unsigned char xAddr;
	unsigned char yAddr;
} GrLcdCtrlrStateType;

typedef struct struct_GrLcdStateType
{
	unsigned char lcdXAddr;
	unsigned char lcdYAddr;
	GrLcdCtrlrStateType ctrlr[2];
} GrLcdStateType;

#include "fontgr.h"		//police semi graphique
#include "font5x7.h"	// police de caractres type ARIAL

// global variables
GrLcdStateType GrLcdState;

/*************************************************************/
/********************** LOCAL FUNCTIONS **********************/
/*************************************************************/

void delay(void)
{
unsigned char c;
for(c=0;c<0x2;c++);
}

void glcdInitHW(void)		//HW au dbut
{
// initialize LCD control port to output
	LCD_DATA_DDR=entre;
	LCD_CTRL_DDR=sort;
	LCD_CTRL_RS=1;
	LCD_CTRL_RW=1;
	LCD_CTRL_E=1;
	LCD_CTRL_CS0=1;
	LCD_CTRL_CS1=1;
	LCD_CTRL_RESET=1;
// initialize LCD control lines levels
	LCD_CTRL_RS=0;
	LCD_CTRL_RW=0;
	LCD_CTRL_E=0;
	LCD_CTRL_CS0=1;
	LCD_CTRL_CS1=1;
	LCD_CTRL_RESET=0;
// initialize LCD data port to output
	LCD_DATA_DDR=sort;
// initialize LCD data
	LCD_DATA_PORT=0x00;
}

void glcdControllerSelect(unsigned char controller)
{
	// select requested controller
	switch(controller)
	{
	case 0:	LCD_CTRL_CS0=1; LCD_CTRL_CS1=0; break;
	case 1:	LCD_CTRL_CS0=0; LCD_CTRL_CS1=1; break;
	default: break;
	}
}

void glcdBusyWait(unsigned char controller)
{
// wait until LCD busy bit goes to zero
	// select the controller chip
	glcdControllerSelect(controller);
	// do a read from control register;
	LCD_DATA_DDR=entre;	
	LCD_CTRL_RS=0;
	LCD_CTRL_RW=1;
	LCD_CTRL_E=1;
	while(LCD_DATA_PORT & GLCD_STATUS_BUSY)
	{
		LCD_CTRL_E=0; Nop(); Nop();	Nop(); Nop();
		LCD_CTRL_E=1; Nop(); Nop();	Nop(); Nop();
	} 
	LCD_CTRL_E=0;
	LCD_CTRL_RW=0;
	LCD_DATA_DDR=sort;	
}

void glcdControlWrite(unsigned char controller, unsigned char data)
{
glcdBusyWait(controller);	// wait until LCD not busy
	LCD_CTRL_RS=0;
	LCD_CTRL_RW=0;
	LCD_DATA_DDR=sort;	
	delay();
	LCD_DATA_PORT=data;
	LCD_CTRL_E=1;
	LCD_CTRL_E=0;
}

void glcdSetXAddress(unsigned char xAddr)
{
	// record address change locally
	GrLcdState.lcdXAddr = xAddr;
	// clear y (col) address on both controllers
	glcdControlWrite(0, GLCD_Y_ADDR | 0x00);
	GrLcdState.ctrlr[0].xAddr = 0;
	glcdControlWrite(1, GLCD_Y_ADDR | 0x00);
	GrLcdState.ctrlr[1].xAddr = 0;
	// set y (col) address on destination controller
	glcdControlWrite((GrLcdState.lcdXAddr/GLCD_CONTROLLER_WIDTH),
		GLCD_Y_ADDR | (GrLcdState.lcdXAddr & 0x3F));
}

void glcdSetYAddress(unsigned char yAddr)
{
	// record address change locally
	GrLcdState.lcdYAddr = yAddr;
	// set page address for both controllers
	glcdControlWrite(0, GLCD_SET_PAGE | yAddr);
	glcdControlWrite(1, GLCD_SET_PAGE | yAddr);
}

void glcdDataWrite(unsigned char data)
{
unsigned char controller;

controller = (GrLcdState.lcdXAddr/GLCD_CONTROLLER_WIDTH);
	glcdBusyWait(controller);		// wait until LCD not busy
	LCD_CTRL_RS=1;
	LCD_CTRL_RW=0;
	LCD_DATA_DDR=sort;	
	delay();
	LCD_DATA_PORT=data;
	LCD_CTRL_E=1;
	LCD_CTRL_E=0;

	// increment our local address counter
	GrLcdState.ctrlr[controller].xAddr++;
	GrLcdState.lcdXAddr++;
	if(GrLcdState.lcdXAddr > GLCD_XPIXELS)
	{
		GrLcdState.lcdYAddr++;
		glcdSetYAddress(GrLcdState.lcdYAddr);
		glcdSetXAddress(0);
	}
}

unsigned char glcdDataRead(void)
{
	unsigned char data;
    unsigned char controller = (GrLcdState.lcdXAddr/GLCD_CONTROLLER_WIDTH);

	glcdBusyWait(controller);		// wait until LCD not busy
	LCD_DATA_DDR=entre;	
	LCD_CTRL_RS=1;
	LCD_CTRL_RW=1;
	delay();
	data = LCD_DATA_PORT;
	LCD_CTRL_E=1;
	LCD_CTRL_E=0;
	data = LCD_DATA_PORT;
	LCD_CTRL_RW=0;
	LCD_DATA_DDR=sort;	
	delay();

	// increment our local address counter
	GrLcdState.ctrlr[controller].xAddr++;
	GrLcdState.lcdXAddr++;
	if(GrLcdState.lcdXAddr > GLCD_XPIXELS)
	{
		GrLcdState.lcdYAddr++;
		glcdSetYAddress(GrLcdState.lcdYAddr);
		glcdSetXAddress(0);
	}
	return data;
}

void glcdReset(unsigned char resetState)
{
	// reset lcd if argument is true
	// run lcd if argument is false
	if(resetState)
		LCD_CTRL_RESET=0;
	else
		LCD_CTRL_RESET=1;
}

/*************************************************************/
/********************* PUBLIC FUNCTIONS **********************/
/*************************************************************/
// dfinit le ligne de dpart
void glcdStartLine(unsigned char start)
{
	glcdControlWrite(0, GLCD_START_LINE | start);
	glcdControlWrite(1, GLCD_START_LINE | start);
}

void glcdSetAddress(unsigned char x, unsigned char yLine)
{
	// set addresses
	glcdSetYAddress(yLine);
	glcdSetXAddress(x);
}

void glcdClearScreen(void)
{
	unsigned char pageAddr;
	unsigned char xAddr;

	// clear LCD
	// loop through all pages
	for(pageAddr=0; pageAddr<(GLCD_YPIXELS>>3); pageAddr++)
	{
		// set page address
		glcdSetAddress(0, pageAddr);
		// clear the 128 lines of this page
		for(xAddr=0; xAddr<GLCD_XPIXELS; xAddr++)
		{
			glcdDataWrite(0x00);
		}
	}
}

void glcdGotoChar(unsigned char line, unsigned char col)
{
	glcdSetAddress(col*6, line);
}

void glcdDelay(unsigned int p)			// 1-8us      ...2-13us     ...5-31us
{								// 10-60us    ...50-290us
	unsigned int i;				// 100-580us  ...500-2,9ms
	unsigned char j; 			// 1000-5,8ms ...5000-29ms
								// 10000-56ms ...30000-170ms
								// 50000-295ms...60000-345ms
	for (i = 0; i < p; i++); for (j = 0; j < 1; j++);
}

void glcdHome(void)
{
	// initialize addresses/positions
	glcdStartLine(0);
	glcdSetAddress(0,0);
	// initialize local data structures
	GrLcdState.ctrlr[0].xAddr = 0;
	GrLcdState.ctrlr[0].yAddr = 0;
	GrLcdState.ctrlr[0].xAddr = 0;
	GrLcdState.ctrlr[0].yAddr = 0;
}

void glcdInit(void)
{
	// initialize hardware
	glcdInitHW();
	// bring lcd out of reset
	glcdReset(FALSE);
	// Turn on LCD
	glcdControlWrite(0, GLCD_ON_CTRL | GLCD_ON_DISPLAY);
	glcdControlWrite(1, GLCD_ON_CTRL | GLCD_ON_DISPLAY);
	// clear lcd
	glcdClearScreen();
	// initialize positions
	glcdHome();
}

////////////////////////////////////////////////////////////////////////////// 
 // graphic routines

// set dot
void glcdSetDot(unsigned char x, unsigned char y)
{
unsigned char temp;

	glcdSetAddress(x, y/8);
	temp = glcdDataRead();	// dummy read
	temp = glcdDataRead();	// read back current value
	glcdSetAddress(x, y/8);
	glcdDataWrite(temp | (1 << (y % 8)));
	glcdStartLine(0);
}

// clear dot
void glcdClearDot(unsigned char x, unsigned char y)
{
unsigned char temp;

	glcdSetAddress(x, y/8);
	temp = glcdDataRead();	// dummy read
	temp = glcdDataRead();	// read back current value
	glcdSetAddress(x, y/8);
	glcdDataWrite(temp & ~(1 << (y % 8)));
	glcdStartLine(0);
}

// draw line 
// x1,y1 et x2,y2 reprsentent les coordonnes des extremes de la droite
// coul sa couleur, 0 efface, 1 affiche les points
void glcdLine(int x1, int y1, int x2, int y2,unsigned char coul)
{
int i,j;
float a,b,c;
	if (x2!=x1) // ligne oblique
	{
		a=((float)y2-(float)y1)/((float)x2-(float)x1);	// recherche de y=ax+b
		b=((float)y1*(float)x2-(float)y2*(float)x1)/((float)x2-(float)x1);
		for(i=x1;i<=x2;i++)
			{
				c=a*(float)i+b;
				j=(int)c;
				if (coul) glcdSetDot(i,j);
				else glcdClearDot(i,j);
			}
	}
	else
	{
	for(i=y1;i<=y2;i++)	
		{
			if (coul) glcdSetDot(x1,i);
			else glcdClearDot(x1,i);
		}
	}
}


// trace un rectangle plein 
void glcdFRectangle(unsigned char x, unsigned char y, unsigned char a, unsigned char b,unsigned char coul)
{
 unsigned char i,j;
if (coul)
  {
  	for (i=0;i<b;i++)
	{
  		for (j = 0; j < a; j++) 
		{
			glcdSetDot(x+i, y+j);
		}
 	}
  }
}

// draw rectangle
void glcdRectangle(unsigned char x, unsigned char y, unsigned char a, unsigned char b,unsigned char coul)
{
  unsigned char j;
if (coul)
  {
  for (j = 0; j < a; j++) {
		glcdSetDot(x, y + j);
		glcdSetDot(x + b - 1, y + j);
	}
  for (j = 0; j < b; j++)	{
		glcdSetDot(x + j, y);
		glcdSetDot(x + j, y + a - 1);
	}
  }
else
  {for (j = 0; j < a; j++) {
		glcdClearDot(x, y + j);
		glcdClearDot(x + b - 1, y + j);
	}
  for (j = 0; j < b; j++)	{
		glcdClearDot(x + j, y);
		glcdClearDot(x + j, y + a - 1);
	}
  
  }
}

// draw circle
void glcdCircle(unsigned char xcenter, unsigned char ycenter, unsigned char radius,unsigned char coul)
{
  int tswitch, y, x = 0;
  unsigned char d;

  d = ycenter - xcenter;
  y = radius;
  tswitch = 3 - 2 * radius;
  if (!coul)
  {
  	while (x <= y) {
    				glcdClearDot(xcenter + x, ycenter + y);     glcdClearDot(xcenter + x, ycenter - y);
    				glcdClearDot(xcenter - x, ycenter + y);     glcdClearDot(xcenter - x, ycenter - y);
    				glcdClearDot(ycenter + y - d, ycenter + x); glcdClearDot(ycenter + y - d, ycenter - x);
    				glcdClearDot(ycenter - y - d, ycenter + x); glcdClearDot(ycenter - y - d, ycenter - x);
				    if (tswitch < 0) tswitch += (4 * x + 6);
    				else {
      						tswitch += (4 * (x - y) + 10);
      						y--;
    					 }
    				x++;
  					}
  }
  else
  {
  	while (x <= y) {
    				glcdSetDot(xcenter + x, ycenter + y);     glcdSetDot(xcenter + x, ycenter - y);
    				glcdSetDot(xcenter - x, ycenter + y);     glcdSetDot(xcenter - x, ycenter - y);
    				glcdSetDot(ycenter + y - d, ycenter + x); glcdSetDot(ycenter + y - d, ycenter - x);
    				glcdSetDot(ycenter - y - d, ycenter + x); glcdSetDot(ycenter - y - d, ycenter - x);
				    if (tswitch < 0) tswitch += (4 * x + 6);
    				else {
      						tswitch += (4 * (x - y) + 10);
      						y--;
    					 }
    				x++;
  					}  
  }
}

//////////////////////////////////////////////////////////////////////////////////////////
// text routines

// write a character at the current position
void glcdWriteChar(char c)
{
	unsigned int i = 0;
	unsigned int j = 0;

	for(i=0; i<5; i++)
	{j=(((unsigned int)c - 0x20) * 5) + i;
		glcdDataWrite(Font5x7[j]);
	}

	// write a spacer line
	glcdDataWrite(0x00);
	glcdStartLine(0);
}

// crit un caractre semi graphique (dans Fontgr
void glcdWriteCharGr(unsigned char grCharIdx)
{
	unsigned char idx;
	unsigned char grLength;
	unsigned char grStartIdx = 0;

	// get starting index of graphic bitmap
	for(idx=0; idx<grCharIdx; idx++)
	{ 
		// add this graphic's length to the startIdx
		// to get the startIdx of the next one
		grStartIdx += FontGr[grStartIdx];
	}
	grLength = FontGr[grStartIdx];

	// write the lines of the desired graphic to the display
	for(idx=0; idx<grLength; idx++)
	{
		// write the line
		glcdDataWrite(FontGr[(grStartIdx+1)+idx]);
	}
}

// criture de chaines : deux fonctions sur PIC (architechture HARVARD)
// crit une chaine de caractres en ROM
void glcdPutStr(rom unsigned char *data)
{
  while (*data) {
    glcdWriteChar(*data);
    data++;
  }
}

// crit une chaine de caractres en RAM 
void glcdPutStrRAM(unsigned char *data)
{
  while (*data) {
    glcdWriteChar(*data);
    data++;
  }
}
//----------------------------------------------------------------------------------------------------------
